@@ -27,7 +27,11 @@
 This should end with a space.")
 
 (defcustom hypb:rgrep-command
-  (format "%sgrep -insIHr" (if (executable-find "zgrep") "z" ""))
+  ;; Only the FreeBSD version of zgrep supports all of the grep
+  ;; options that Hyperbole needs: -r, --include, and --exclude
+  (format "%sgrep -insIHr" (if (and (executable-find "zgrep")
+                                    (string-match-p "bsd" (shell-command-to-string "zgrep --version | head -1")))
+                               "z" ""))
   "*Grep command string and initial arguments to send to `hypb:rgrep' command.
 It must end with a space."
   :type 'string
@@ -38,7 +42,7 @@ It must end with a space."
 ;;; ************************************************************************
 
 (defun hypb:call-process-p (program &optional infile predicate &rest args)
-  "Calls an external PROGRAM with INFILE for input.
+  "Call an external PROGRAM with INFILE for input.
 If PREDICATE is given, it is evaluated in a buffer with the PROGRAM's
 output and the result returned.  If PREDICATE is nil, returns t iff
 program has no output or just a 0-valued output.
@@ -49,7 +53,7 @@ Rest of ARGS are passed as arguments to PROGRAM."
       (setq buffer-read-only nil)
       (erase-buffer)
       (apply 'call-process program infile buf nil args)
-      (setq found 
+      (setq found
 	    (if predicate
 		(eval predicate)
 	      (or (= (point-max) 1) ;; No output, consider cmd a success.
@@ -64,12 +68,12 @@ Rest of ARGS are passed as arguments to PROGRAM."
   "Return count of occurrences of CHAR in ARRAY."
   (let ((i 0) (c 0) (l (length array)))
     (while (< i l)
-      (if (= char (aref array i)) (setq c (1+ c)))
+      (when (= char (aref array i)) (setq c (1+ c)))
       (setq i (1+ i)))
     c))
 
 (defun hypb:chmod (op octal-permissions file)
-  "Uses OP and OCTAL-PERMISSIONS integer to set FILE permissions.
+  "Use OP and OCTAL-PERMISSIONS integer to set FILE permissions.
 OP may be +, -, xor, or default =."
   (let ((func (cond ((eq op '+)   #'logior)
 		    ((eq op '-)   (lambda (p1 p2) (logand (lognot p1) p2)))
@@ -79,20 +83,20 @@ OP may be +, -, xor, or default =."
 				  (file-modes file)))))
 
 (defun hypb:cmd-key-string (cmd-sym &optional keymap)
-  "Returns a single pretty printed key sequence string bound to CMD-SYM.
+  "Return a single pretty printed key sequence string bound to CMD-SYM.
 Global keymap is used unless optional KEYMAP is given."
   (if (and cmd-sym (symbolp cmd-sym) (fboundp cmd-sym))
-  (let* ((get-keys (lambda (cmd-sym keymap)
-		     (key-description (where-is-internal
-				       cmd-sym keymap 'first))))
-	 (keys (funcall get-keys cmd-sym keymap)))
-    (concat "{"
-	    (if (string= keys "")
-		(concat (funcall get-keys 'execute-extended-command nil)
-			" " (symbol-name cmd-sym) " RET")
-	      keys)
-	    "}"))
-  (error "(hypb:cmd-key-string): Invalid cmd-sym arg: %s." cmd-sym)))
+      (let* ((get-keys (lambda (cmd-sym keymap)
+		         (key-description (where-is-internal
+				           cmd-sym keymap 'first))))
+	     (keys (funcall get-keys cmd-sym keymap)))
+        (concat "{"
+	        (if (string= keys "")
+		    (concat (funcall get-keys 'execute-extended-command nil)
+			    " " (symbol-name cmd-sym) " RET")
+	          keys)
+	        "}"))
+    (error "(hypb:cmd-key-string): Invalid cmd-sym arg: %s" cmd-sym)))
 
 ;;;###autoload
 (defun hypb:configuration (&optional out-buf)
@@ -111,40 +115,39 @@ Global keymap is used unless optional KEYMAP is given."
 			    (t (hypb:replace-match-string
 				" of .+" (emacs-version) "" t)))
                       hyperb:version))
-      (if (and (boundp 'br-version) (stringp br-version))
-	  (insert (format "\tOO-Browser:  %s\n" br-version)))
-      (if (and (boundp 'system-configuration) (stringp system-configuration))
-	  (insert (format "\tSys Type:    %s\n" system-configuration)))
+      (when (and (boundp 'br-version) (stringp br-version))
+	(insert (format "\tOO-Browser:  %s\n" br-version)))
+      (when (and (boundp 'system-configuration) (stringp system-configuration))
+	(insert (format "\tSys Type:    %s\n" system-configuration)))
       (insert (format "\tOS Type:     %s\n\tWindow Sys:  %s\n"
                       system-type (or window-system (hyperb:window-system)
 				      "None")))
-      (if (and (boundp 'hmail:reader) hmail:reader)
-          (insert (format "\tMail Reader: %s\n"
-                          (cond ((eq hmail:reader 'rmail-mode) "RMAIL")
-                                ((eq hmail:reader 'vm-mode)
-                                 (concat "VM " vm-version))
-                                ((and (eq hmail:reader 'mh-show-mode)
-                                      (string-match "v ?\\([0-9]+.[0-9]+\\)"
-                                          mh-e-RCS-id))
-                                 (concat "MH-e "
-                                         (substring mh-e-RCS-id
-                                                    (match-beginning 1)
-                                                    (match-end 1))))
-                                ((eq hmail:reader 'pm-fdr-mode)
-                                 (concat "PIEmail " pm-version))
-                                ))))
-      (if (and (boundp 'hnews:reader) (boundp 'gnus-version) hnews:reader)
-          (insert (format "\tNews Reader: %s\n" gnus-version)))
+      (when (and (boundp 'hmail:reader) hmail:reader)
+        (insert (format "\tMail Reader: %s\n"
+                        (cond ((eq hmail:reader 'rmail-mode) "RMAIL")
+                              ((eq hmail:reader 'vm-mode)
+                               (concat "VM " vm-version))
+                              ((and (eq hmail:reader 'mh-show-mode)
+                                    (string-match "v ?\\([0-9]+.[0-9]+\\)"
+                                                  mh-e-RCS-id))
+                               (concat "MH-e "
+                                       (substring mh-e-RCS-id
+                                                  (match-beginning 1)
+                                                  (match-end 1))))
+                              ((eq hmail:reader 'pm-fdr-mode)
+                               (concat "PIEmail " pm-version))))))
+      (when (and (boundp 'hnews:reader) (boundp 'gnus-version) hnews:reader)
+        (insert (format "\tNews Reader: %s\n" gnus-version)))
       (insert "\n")
       ;; Insert recent Hyperbole debugging messages if any.
-      (if (get-buffer "*Messages*")
-	  (let ((opoint (point)))
-	    (insert-buffer-substring "*Messages*")
-	    (keep-lines "^(HyDebug)" opoint (point))))
+      (when (get-buffer "*Messages*")
+	(let ((opoint (point)))
+	  (insert-buffer-substring "*Messages*")
+	  (keep-lines "^(HyDebug)" opoint (point))))
       (untabify start (point)))))
 
 (defun hypb:debug ()
-  "Loads Hyperbole hbut.el source file and sets debugging traceback flag."
+  "Load Hyperbole hbut.el source file and set debugging traceback flag."
   (interactive)
   (or (featurep 'hinit) (load "hyperbole"))
   (or (and (featurep 'hbut)
@@ -182,45 +185,46 @@ Global keymap is used unless optional KEYMAP is given."
       string)))
 
 (defun hypb:domain-name ()
-  "Returns current Internet domain name with '@' prepended or nil if none."
+  "Return current Internet domain name with '@' prepended or nil if none."
   (let* ((dname-cmd (or (file-exists-p "/usr/bin/domainname")
 			(file-exists-p "/bin/domainname")))
 	 (dname (or (and (boundp 'message-user-fqdn) (stringp message-user-fqdn)
 			 (string-match "\\." message-user-fqdn)
 			 message-user-fqdn)
 		    (getenv "DOMAINNAME")
-		    (if dname-cmd
-			(hypb:call-process-p
-			 "domainname" nil 
-			 '(substring (buffer-string) 0 -1)))))
+		    (when dname-cmd
+		      (hypb:call-process-p
+		       "domainname" nil
+		       '(substring (buffer-string) 0 -1)))))
 	 host-and-domain)
-    (if (or (and dname (string-match "\\." dname))
-	    (and (setq host-and-domain (hypb:call-process-p
-					"hostname" nil '(substring (buffer-string) 0 -1) "-f"))
-		 (setq dname (if (string-match "\\`[^.]+\\." host-and-domain)
+    (when (or (and dname (string-match "\\." dname))
+	      (and (setq host-and-domain (hypb:call-process-p
+					  "hostname" nil '(substring (buffer-string) 0 -1) "-f"))
+		   (setq dname (when (string-match "\\`[^.]+\\." host-and-domain)
 				 (substring host-and-domain (match-end 0)))))
-	    (let* ((src "/etc/resolv.conf")
-		   (src-buf-exists-p (get-file-buffer src)))
-	      (and (file-exists-p src) (file-readable-p src)
-		   (with-temp-buffer
-		     (insert-file-contents-literally src)
-		     (goto-char (point-min))
-		     (if (re-search-forward  "^domain[ \t]+\\([^ \t\n\r]+\\)"
-					     nil t)
-			 (setq dname (buffer-substring (match-beginning 1)
-						       (match-end 1))))
-		     (or src-buf-exists-p (kill-buffer nil))
-		     dname))))
-	(concat "@" dname))))
+	      (let* ((src "/etc/resolv.conf")
+		     (src-buf-exists-p (get-file-buffer src)))
+	        (and (file-exists-p src) (file-readable-p src)
+		     (with-temp-buffer
+		       (insert-file-contents-literally src)
+		       (goto-char (point-min))
+		       (when (re-search-forward  "^domain[ \t]+\\([^ \t\n\r]+\\)" nil t)
+			 (setq dname (match-string 1)))
+		       (or src-buf-exists-p (kill-buffer nil))
+		       dname))))
+      (concat "@" dname))))
 
 (defun hypb:emacs-byte-code-p (obj)
   "Return non-nil iff OBJ is an Emacs byte compiled object."
-  (or (and (fboundp 'compiled-function-p) (compiled-function-p obj))
-      (and (fboundp 'byte-code-function-p) (byte-code-function-p obj))))
+  (or (and (fboundp 'byte-code-function-p) (byte-code-function-p obj))
+      (and (fboundp 'compiled-function-p) (compiled-function-p obj))))
 
 (defun hypb:error (&rest args)
-  "Signals an error typically to be caught by `hyperbole'."
-  (let ((msg (if (< (length args) 2) (car args) (apply 'format args))))
+  "Signal an error typically to be caught by `hyperbole'."
+  (let ((msg (if (< (length args) 2)
+		 (car args)
+	       (apply 'format (cons (car args)
+				    (mapcar #'hypb:format-quote (cdr args)))))))
     (put 'error 'error-message msg)
     (error msg)))
 
@@ -233,21 +237,23 @@ FILE is temporarily read into a buffer to determine the major mode if necessary.
       (unless (or existing-flag (null buf))
 	(kill-buffer buf)))))
 
-(defun hypb:format-quote (string)
-  "Replace all single % with %% in STRING so a call to `format' or `message' ignores them."
-  (if (stringp string)
+(defun hypb:format-quote (arg)
+  "Replace all single % with %% in any string ARG so that a call to `format' or `message' ignore them.
+Return either the modified string or the original ARG."
+  (if (stringp arg)
       (replace-regexp-in-string
        "@@@" "%%" (replace-regexp-in-string
-		   "%" "%%" (replace-regexp-in-string "%%" "@@@" string nil t)
+		   "%" "%%" (replace-regexp-in-string "%%" "@@@" arg nil t)
 		   nil t)
-       nil t)))
+       nil t)
+    arg))
 
 (defun hypb:function-copy (func-symbol)
-  "Copies FUNC-SYMBOL's body for overloading.  Returns copy of body."
+  "Copy FUNC-SYMBOL's body for overloading.  Return copy of body."
   (if (fboundp func-symbol)
       (let ((func (hypb:indirect-function func-symbol)))
 	(cond ((listp func) (copy-sequence func))
-	      ((subrp func) (error "(hypb:function-copy): `%s' is a primitive; can't copy body."
+	      ((subrp func) (error "(hypb:function-copy): `%s' is a primitive; can't copy body"
 				   func-symbol))
 	      ((and (hypb:emacs-byte-code-p func) (fboundp 'make-byte-code))
 	       (if (not (fboundp 'compiled-function-arglist))
@@ -262,11 +268,11 @@ FILE is temporarily read into a buffer to determine the major mode if necessary.
 					(compiled-function-stack-depth func)
 					(compiled-function-doc-string func))))
 		       spec)
-		   (if (setq spec (compiled-function-interactive func))
-		       (setq new-code (nconc new-code (list (nth 1 spec)))))
+		   (when (setq spec (compiled-function-interactive func))
+		     (setq new-code (nconc new-code (list (nth 1 spec)))))
 		   (apply 'make-byte-code new-code))))
 	      (t (error "(hypb:function-copy): Can't copy function body: %s" func))))
-    (error "(hypb:function-copy): `%s' symbol is not bound to a function."
+    (error "(hypb:function-copy): `%s' symbol is not bound to a function"
 	   func-symbol)))
 
 (defun hypb:function-overload (func-sym prepend &rest new-forms)
@@ -296,44 +302,47 @@ FILE is temporarily read into a buffer to determine the major mode if necessary.
 		(append old-func-call new-forms)))))))
 
 (defun hypb:function-symbol-replace (func-sym sym-to-replace replace-with-sym)
-  "Replaces in body of FUNC-SYM SYM-TO-REPLACE with REPLACE-WITH-SYM.
+  "Replace in body of FUNC-SYM SYM-TO-REPLACE with REPLACE-WITH-SYM.
 FUNC-SYM may be a function symbol or its body.  All occurrences within lists
 are replaced.  Returns body of modified FUNC-SYM."
   (let ((body (hypb:indirect-function func-sym))
 	(constant-vector) (constant))
-    (if (listp body)
-	;; assume V18 byte compiler
-	(setq constant-vector
-	      (car (delq nil (mapcar
-			      (lambda (elt)
-				(and (listp elt)
-				     (vectorp (setq constant-vector (nth 2 elt)))
-				     constant-vector))
-			      body))))
-      ;; assume EMACS byte compiler   (eq (compiled-function-p body) t)
-      (setq constant (if (fboundp 'compiled-function-constants)
-			 (compiled-function-constants body)
-		       (aref body 2))
-	    constant-vector (if (vectorp constant) constant)))
-    (if constant-vector
-	;; Code is byte-compiled.
-	(hypb:constant-vector-symbol-replace
-	 constant-vector sym-to-replace replace-with-sym)
-      ;;
-      ;; Code is not byte-compiled.
-      ;; Replaces occurrence of symbol within lists only.
-      (hypb:map-sublists
-       (lambda (atom list)
-	 ;; The ' in the next line *is* required for proper substitution.
-	 (if (eq atom 'sym-to-replace)
+    (if (subrp body)
+        ;; Non-Lisp code, can't do any replacement
+        body
+      (if (listp body)
+	  ;; assume V18 byte compiler
+	  (setq constant-vector
+	        (car (delq nil (mapcar
+			        (lambda (elt)
+				  (and (listp elt)
+				       (vectorp (setq constant-vector (nth 2 elt)))
+				       constant-vector))
+			        body))))
+        ;; assume EMACS byte compiler   (eq (compiled-function-p body) t)
+        (setq constant (if (fboundp 'compiled-function-constants)
+			   (compiled-function-constants body)
+		         (aref body 2))
+	      constant-vector (when (vectorp constant) constant)))
+      (if constant-vector
+	  ;; Code is byte-compiled.
+	  (hypb:constant-vector-symbol-replace
+	   constant-vector sym-to-replace replace-with-sym)
+        ;;
+        ;; Code is not byte-compiled.
+        ;; Replaces occurrence of symbol within lists only.
+        (hypb:map-sublists
+         (lambda (atom list)
+	   ;; The ' in the next line *is* required for proper substitution.
+	   (when (eq atom 'sym-to-replace)
 	     (let ((again t))
 	       (while (and again list)
-		 (if (eq (car list) atom)
+	         (if (eq (car list) atom)
 		     (progn (setcar list replace-with-sym)
 			    (setq again nil))
 		   (setq list (cdr list)))))))
-       body))
-    body))
+         body))
+      body)))
 
 ;; Derived from pop-global-mark of "simple.el" in GNU Emacs.
 (defun hypb:goto-marker (marker)
@@ -355,7 +364,7 @@ If MARKER is invalid signal an error."
 	     (switch-to-buffer buffer)))))
 
 (defun hypb:help-buf-name (&optional suffix)
-  "Returns a Hyperbole help buffer name for current buffer.
+  "Return a Hyperbole help buffer name for current buffer.
 With optional SUFFIX string, uses it rather than buffer name."
   (let ((bn (or suffix (buffer-name))))
     (if (string-match (regexp-quote hypb:help-buf-prefix) bn)
@@ -412,7 +421,7 @@ copied, otherwise, it is omitted."
 	
 ;;;###autoload
 (defun hypb:locate (search-string &optional filter arg)
-  "Find file name matches anywhere, calling the value of `locate-command', and putting results in the `*Locate*' buffer.
+  "Find file name match anywhere, calling the value of `locate-command', and putting results in the `*Locate*' buffer.
 Pass it SEARCH-STRING as argument.  Interactively, prompt for SEARCH-STRING.
 With prefix arg ARG, prompt for the exact shell command to run instead.
 
@@ -444,23 +453,21 @@ then `locate-post-command-hook'."
 
 (defalias 'hypb:mark #'mark)
 
-(if (featurep 'xemacs)
-    (defalias 'hypb:mark-marker 'mark-marker)
-  (defun hypb:mark-marker (inactive-p)
-    "Return this buffer's mark as a marker object, or nil if no mark.
+(defun hypb:mark-marker (inactive-p)
+  "Return this buffer's mark as a marker object, or nil if no mark.
 INACTIVE-P is unused, it is for compatibility with XEmacs' version of
-mark-marker."
-    (mark-marker)))
+`mark-marker'."
+  (mark-marker))
 
 ;;;###autoload
 (defun hypb:map-plist (func plist)
-  "Returns result of applying FUNC of two args, key and value, to key-value pairs in PLIST, a property list."
+  "Return result of applying FUNC of two args, key and value, to key-value pairs in PLIST, a property list."
   (cl-loop for (k v) on plist by #'cddr
 	   collect (funcall func k v) into result
 	   finally return result))
 
 (defun hypb:map-sublists (func list)
-  "Applies FUNC to every atom found at any level of LIST.
+  "Apply FUNC to every atom found at any level of LIST.
 FUNC must take two arguments, an atom and a list in which the atom is found.
 Returns values from applications of FUNC as a list with the same
 structure as LIST.  FUNC is therefore normally used just for its side-effects."
@@ -471,10 +478,10 @@ structure as LIST.  FUNC is therefore normally used just for its side-effects."
 	  list))
 
 (defun hypb:map-vector (func object)
-  "Returns list of results of application of FUNC to each element of OBJECT.
-OBJECT should be a vector or byte-code object."
-  (if (not (or (vectorp object) (hypb:emacs-byte-code-p object)))
-      (error "(hypb:map-vector): Second argument must be a vector or byte-code object."))
+  "Return list of results of application of FUNC to each element of OBJECT.
+OBJECT should be a vector or `byte-code' object."
+  (unless (or (vectorp object) (hypb:emacs-byte-code-p object))
+    (error "(hypb:map-vector): Second argument must be a vector or byte-code object"))
   (let ((end (length object))
 	(i 0)
 	(result))
@@ -483,6 +490,14 @@ OBJECT should be a vector or byte-code object."
 	    i (1+ i)))
     (nreverse result)))
 
+(defun hypb:mark-object (object)
+  "Mark OBJECT as a Hyperbole object if possible to prevent generic functions from changing it.
+OBJECT must be a non-empty string or a symbol or this has no effect."
+  (cond ((and (stringp object) (not (string-empty-p object)))
+	 (put-text-property 0 1 'hyperbole t object))
+	((symbolp object)
+	 (put object 'hyperbole t))))
+
 ;; Derived from "window.el".
 (defun hypb:maximize-window-height (&optional window)
   "Maximize WINDOW.
@@ -497,8 +512,15 @@ WINDOW pixelwise."
    window (window-max-delta window nil nil nil nil nil window-resize-pixelwise)
    nil nil window-resize-pixelwise))
 
+(defun hypb:object-p (object)
+  "Return t if OBJECT is marked as a Hyperbole object, else nil."
+  (cond ((and (stringp object) (not (string-empty-p object)))
+	 (get-text-property 0 'hyperbole object))
+	((symbolp object)
+	 (get object 'hyperbole))))
+
 (defun hypb:replace-match-string (regexp str newtext &optional literal)
-  "Replaces all matches for REGEXP in STR with NEWTEXT string and returns the result.
+  "Replace all matches for REGEXP in STR with NEWTEXT string and return the result.
 Optional LITERAL non-nil means do a literal replacement.
 Otherwise treat \\ in NEWTEXT string as special:
   \\& means substitute original matched text,
@@ -511,49 +533,10 @@ that returns a replacement string."
   (unless (or (stringp newtext) (functionp newtext))
     (error "(hypb:replace-match-string): 3rd arg must be a string or function: %s"
 	   newtext))
-  (let ((rtn-str "")
-	(start 0)
-	(special)
-	match prev-start)
-    (while (setq match (string-match regexp str start))
-      (setq prev-start start
-	    start (match-end 0)
-	    rtn-str
-	    (concat
-	      rtn-str
-	      (substring str prev-start match)
-	      (cond ((functionp newtext)
-		     (hypb:replace-match-string
-		      regexp (substring str match start)
-		      (funcall newtext str) literal))
-		    (literal newtext)
-		    (t (mapconcat
-			 (lambda (c)
-			   (if special
-			       (progn
-				 (setq special nil)
-				 (cond ((eq c ?\\) "\\")
-				       ((eq c ?&)
-					(substring str
-						   (match-beginning 0)
-						   (match-end 0)))
-				       ((and (>= c ?0) (<= c ?9))
-					(if (> c (+ ?0 (length
-							(match-data))))
-					    ;; Invalid match num
-					    (error "(hypb:replace-match-string) Invalid match num: %c" c)
-					  (setq c (- c ?0))
-					  (substring str
-						     (match-beginning c)
-						     (match-end c))))
-				       (t (char-to-string c))))
-			     (if (eq c ?\\) (progn (setq special t) nil)
-			       (char-to-string c))))
-			 newtext ""))))))
-    (concat rtn-str (substring str start))))
+  (replace-regexp-in-string regexp newtext str nil literal))
 
 (defun hypb:return-process-output (program &optional infile &rest args)
-  "Returns as a string the output from external PROGRAM with INFILE for input.
+  "Return as a string the output from external PROGRAM with INFILE for input.
 Rest of ARGS are passed as arguments to PROGRAM.
 Removes any trailing newline at the end of the output."
   (let ((buf (get-buffer-create "*test-output*"))
@@ -564,13 +547,14 @@ Removes any trailing newline at the end of the output."
       (apply 'call-process program infile buf nil args)
       (setq output (buffer-string))
       ;; Remove trailing newline from output.
-      (if (> (length output) 0) (setq output (substring output 0 -1)))
+      (when (> (length output) 0)
+        (setq output (substring output 0 -1)))
       (set-buffer-modified-p nil)
       (kill-buffer buf))
     output))
 
 (defun hypb:remove-lines (regexp)
- "Remove lines containing matches for REGEXP within the active region or to the end of buffer."
+ "Remove lines containing match for REGEXP within the active region or to the end of buffer."
     (interactive "sRemove lines with match for regexp: ")
     (flush-lines regexp nil nil t))
 
@@ -581,7 +565,7 @@ If in an Emacs Lisp mode buffer and no PREFIX-ARG is given, limit search to only
   (interactive (list (if (and (not current-prefix-arg) (equal (buffer-name) "*Locate*"))
 			 (read-string "Grep files listed here for: ")
 		       (let ((default (symbol-at-point)))
-			 (if default (setq default (symbol-name default)))
+			 (when default (setq default (symbol-name default)))
 			 (read-string (format "Rgrep below current dir for%s: "
 					      (if default
 						  (format " (default %s)" default)
@@ -611,7 +595,7 @@ If in an Emacs Lisp mode buffer and no PREFIX-ARG is given, limit search to only
     (grep grep-cmd)))
 
 (defun hypb:save-lines (regexp)
- "Save only lines containing matches for REGEXP within the active region or to the end of buffer."
+ "Save only lines containing match for REGEXP within the active region or to the end of buffer."
     (interactive "sSave lines with match for regexp: ")
     (keep-lines regexp nil nil t))
 
@@ -630,25 +614,25 @@ The value returned is the value of the last form in BODY."
     (error "(hypb:select-window-frame): Argument must be a live window, not '%s'" window)))
 
 (defun hypb:supercite-p ()
-  "Returns non-nil iff the Emacs add-on supercite package is in use."
+  "Return non-nil iff the Emacs add-on supercite package is in use."
   (let (hook-val)
-    (if (memq t (mapcar
-		 (lambda (hook-var)
-		   (and (boundp hook-var)
-			(progn (setq hook-val (symbol-value hook-var))
-			       (cond ((listp hook-val)
-				      (if (memq 'sc-cite-original hook-val)
+    (when (memq t (mapcar
+		   (lambda (hook-var)
+		     (and (boundp hook-var)
+			  (progn (setq hook-val (symbol-value hook-var))
+			         (cond ((listp hook-val)
+				        (when (memq 'sc-cite-original hook-val)
 					  t))
-				     ((eq hook-val 'sc-cite-original))))))
-		 '(mail-citation-hook mail-yank-hooks)))
-	t)))
+				       ((eq hook-val 'sc-cite-original))))))
+		   '(mail-citation-hook mail-yank-hooks)))
+      t)))
 
 (defun hypb:toggle-isearch-invisible (&optional arg)
   "Toggle interactive invisible searching on or off.
 This determines whether to search inside invisible text or not.
 Toggles the variable ‘isearch-invisible’ between values
 nil and a non-nil value of the option ‘search-invisible’
-(or ‘open’ if ‘search-invisible’ is nil).
+\(or ‘open’ if ‘search-invisible’ is nil).
 
 With optional prefix ARG > 0, turn on searching invisible text.
 If ARG <= 0, turn search only visible text."
@@ -669,7 +653,7 @@ If ARG <= 0, turn search only visible text."
     (user-login-name)))
 
 (defun hypb:window-list (&optional minibuffer-flag)
-  "Returns a list of Lisp window objects for all Emacs windows in selected frame.
+  "Return a list of Lisp window objects for all Emacs windows in selected frame.
 Optional first arg MINIBUFFER-FLAG t means include the minibuffer window
 in the list, even if it is not active.  If MINIBUFFER-FLAG is neither t
 nor nil it means to not count the minibuffer window even if it is active."
@@ -680,36 +664,29 @@ nor nil it means to not count the minibuffer window even if it is active."
 ;;; ************************************************************************
 
 (defvar hypb:home-page "https://www.gnu.org/software/hyperbole/"
-  "The web home page for Hyperbole")
+  "The web home page for Hyperbole.")
 
 (defvar hypb:hyperbole-banner-keymap
   (let ((map (make-sparse-keymap)))
-    (cond ((not (featurep 'xemacs))
-	   (define-key map [mouse-1]  'hypb:browse-home-page)
-	   (define-key map [mouse-2]  'hypb:browse-home-page)
-	   (define-key map "\C-m"     'hypb:browse-home-page))
-	  ((featurep 'xemacs)
-	   (define-key map 'button1  'hypb:browse-home-page)
-	   (define-key map 'button2  'hypb:browse-home-page)
-	   (define-key map '(return) 'hypb:browse-home-page)))
+    (define-key map [mouse-1]  'hypb:browse-home-page)
+    (define-key map [mouse-2]  'hypb:browse-home-page)
+    (define-key map "\C-m"     'hypb:browse-home-page)
     map)
   "Keymap used when on the Hyperbole banner glyph.")
 
 ;;;###autoload
-(defun hypb:display-file-with-logo (&optional file)
-  "Display an optional text FILE in help mode with the Hyperbole banner prepended.
-Without file, the banner is prepended to the current buffer."
-  (if file
-      ;; A stub for this function is defined in hversion.el when not running in InfoDock.
-      (id-browse-file file))
-  (if (not (featurep 'xemacs))
+(defun hypb:display-file-with-logo (file)
+  "Display a text FILE in help mode with the Hyperbole banner prepended."
+  (let ((existing-buf (when (stringp file) (get-file-buffer file))))
+    ;; A stub for this function is defined in hversion.el when not running in InfoDock.
+    (id-browse-file file)
+    (unless existing-buf
       (hypb:display-file-with-logo-emacs file)
-    (hypb:display-file-with-logo-xemacs file))
-  (goto-char (point-min))
-  (skip-syntax-forward "-")
-  (set-window-start (selected-window) 1)
-  (set-buffer-modified-p nil)
-  (help-mode))
+      (goto-char (point-min))
+      (skip-syntax-forward "-")
+      (set-window-start (selected-window) 1)
+      (set-buffer-modified-p nil)
+      (help-mode))))
 
 (defun hypb:browse-home-page ()
   "Visit the web home page for Hyperbole."
@@ -723,7 +700,7 @@ Without file, the banner is prepended to the current buffer."
 
 (defun hypb:constant-vector-symbol-replace
   (constant-vector sym-to-replace replace-with-sym)
-  ;; Replace symbols within a byte-compiled constant vector.
+  "Replace symbols within a byte-compiled constant vector."
   (let ((i (length constant-vector))
 	constant)
     (while (>= (setq i (1- i)) 0)
@@ -739,11 +716,11 @@ Without file, the banner is prepended to the current buffer."
   "Display an optional text FILE with the Hyperbole banner prepended.
 Without file, the banner is prepended to the current buffer."
   (let ((hyperbole-banner-path (expand-file-name "hyperbole-banner.png" hyperb:dir)))
-    (if (not (file-readable-p hyperbole-banner-path))
-	(setq hyperbole-banner-path (if (fboundp 'locate-data-file)
-					(locate-data-file "hyperbole-banner.png")
-				      (expand-file-name "hyperbole-banner.png"
-							data-directory))))
+    (unless (file-readable-p hyperbole-banner-path)
+      (setq hyperbole-banner-path (if (fboundp 'locate-data-file)
+				      (locate-data-file "hyperbole-banner.png")
+				    (expand-file-name "hyperbole-banner.png"
+						      data-directory))))
     (if (or (not (fboundp 'create-image))
 	    (not (display-graphic-p))
 	    (let ((button (next-button (point-min))))
@@ -766,37 +743,6 @@ Without file, the banner is prepended to the current buffer."
 	(button-put button 'face 'default)
 	(button-put button 'keymap hypb:hyperbole-banner-keymap)))))
 
-(defun hypb:display-file-with-logo-xemacs (&optional file)
-  "Display an optional text FILE with the Hyperbole banner prepended.
-Without file, the banner is prepended to the current buffer."
-  (let ((hyperbole-banner-path (expand-file-name "hyperbole-banner.png" hyperb:dir)))
-    (if (not (file-readable-p hyperbole-banner-path))
-	(setq hyperbole-banner-path (if (fboundp 'locate-data-file)
-					(locate-data-file "hyperbole-banner.png")
-				      (expand-file-name "hyperbole-banner.png"
-							data-directory))))
-    (if (or (not (fboundp 'make-glyph))
-	    (let ((extent (next-extent (current-buffer))))
-	      (and extent (extent-property extent 'hyperbole-banner)))
-	    (not hyperbole-banner-path)
-	    (not (file-readable-p hyperbole-banner-path)))
-	;; Either image support is unavailable, the file cannot be read
-	;; or the image has already been inserted, so don't reinsert it.
-	nil
-      (let ((hyperbole-banner (make-glyph hyperbole-banner-path))
-	     (buffer-read-only)
-	     extent)
-	(goto-char (point-min))
-	(insert "\n")
-	(indent-to (startup-center-spaces hyperbole-banner))
-	(insert "\n\n")
-	(setq extent (make-extent (- (point) 3) (- (point) 2)))
-	(set-extent-end-glyph extent hyperbole-banner)
-	(set-extent-property extent 'hyperbole-banner t)
-	(set-extent-property extent 'help-echo
-			     (concat "Click to visit " hypb:home-page))
-	(set-extent-property extent 'keymap hypb:hyperbole-banner-keymap)))))
-
 (defun hypb:locate-pathnames ()
   (save-excursion
     (goto-char (point-min))
@@ -805,15 +751,15 @@ Without file, the banner is prepended to the current buffer."
 			      (buffer-substring-no-properties (point) (point-max)))))
 
 (defun hypb:oct-to-int (oct-num)
-  "Returns octal integer OCTAL-NUM converted to a decimal integer."
+  "Return octal integer OCT-NUM converted to a decimal integer."
   (let ((oct-str (int-to-string oct-num))
 	(dec-num 0))
     (and (string-match "[^0-7]" oct-str)
 	 (error "(hypb:oct-to-int): Bad octal number: %s" oct-str))
     (mapconcat (lambda (o)
 		 (setq dec-num (+ (* dec-num 8)
-				  (if (and (>= o ?0) (<= o ?7))
-				      (- o ?0)))))
+				  (when (and (>= o ?0) (<= o ?7))
+				    (- o ?0)))))
 	       oct-str "")
     dec-num))
 
@@ -821,7 +767,7 @@ Without file, the banner is prepended to the current buffer."
 ;;; Private variables
 ;;; ************************************************************************
 
-(if (not (featurep 'xemacs)) (define-button-type 'hyperbole-banner))
+(define-button-type 'hyperbole-banner)
 
 (provide 'hypb)
 
